/*
 * listener.h
 *
 * Function declarations for listener.c
 *
 * Copyright (c) 2004 Todd MacDermid <tmacd@synacklabs.net>
 *                    nash e.f. <nash@solace.net>
 *
 */

#ifndef CUTLASS_AUDIO_TYPES_H
#define CUTLASS_AUDIO_TYPES_H

/*
 * Cutlass Audio Low-level Driver Types
 */
#define CUTLASS_AUDRIVER_NULL                   0x00
#define CUTLASS_AUDRIVER_ALSA                   0x01
#define CUTLASS_AUDRIVER_OSS                    0x02

/*
 * Cutlass Encoder Driver Types
 */
#define CUTLASS_ENCODER_NULL                    0x01
#define CUTLASS_ENCODER_SPEEX                   0x02

/*
 * Cutlass network ring buffer defaults
 */
#define CUTLASS_AUDIO_RING_SIZE                 32768
#define CUTLASS_AUDIO_RING_MBRSZ                sizeof(float)

#define CUT_AUDIO_PACKET_MAX                    1400
#define CUT_AUDIO_FRAMESIZE_MAX                 1024

/*
 * Cutlass audio sleep timer settings
 */
#define CUTLASS_AUDIO_SLEEP_TV_SEC              0
#define CUTLASS_AUDIO_SLEEP_TV_NSEC             100000

/*
 * Number of under-threshold captures to require before
 * cutting off transmission.
 */

#define CUTLASS_QUIET_FRAMES                    40

/*
 * Cutlass audio blocksize (hardcoded for now)
 */

#define CUTLASS_AUDIO_BLOCKSIZE                 128
#define CUTLASS_AUDIO_SAMPLE_RATE               8000
#define CUTLASS_SPEEX_BLOCKSIZE                 160

/*
 * this structure holds the audio handle, defined in the driver's
 * header file (i.e., audriver-oss.h at the moment).
 *
 * Each connection has its own audio encoder send and receive
 * handles, stored in the connection state.
 *
 */

typedef struct cutlass_audio
{
  int  driver_type;                   /// specifies which audio driver
  void *driver;                       /// pointer to the audio driver's goo
  pthread_mutex_t read_mutex;         /* lock for audio capture driver */
  pthread_mutex_t write_mutex;        /* lock for audio play driver */
  float out_threshold;
} cutlass_audio_t;

#define CUT_AUDIO_SEQUENCE_LEN         4

typedef struct cutlass_conn_audio
{
  int encoder_type;
  int in_framesize;
  int out_framesize;
  uint32_t in_seq_num;
  uint32_t out_seq_num;
  void *decoder_handle;
  void *encoder_handle;
  cutlass_ring_t *out_ring;           /// network ring buffer for outbound
  cutlass_ring_t *in_ring;            /// network ring buffer for inbound
} cutlass_conn_audio_t;

/*
 * Cutlass Audio Message definitions
 */

/* audio request types */
#define CUTLASS_AUDIO_REQ_OPEN              0
#define CUTLASS_AUDIO_REQ_ACCEPT            1
#define CUTLASS_AUDIO_REQ_RESET             2

/* Reset reasoning flags */
#define CUTLASS_RESET_PERM                  0 /* Perm denied */
#define CUTLASS_RESET_ENCODER               1 /* No common encoders */
#define CUTLASS_RESET_BADSESSION            2 /* Spurious packet */
#define CUTLASS_RESET_SESSIONEND            3 /* Normal session termination */

/* audio request flags */

#define CUTLASS_AUDIO_REQ_IWILLSEND         2
#define CUTLASS_AUDIO_REQ_IWILLRECV         4

/* connection audio state types */
#define CUTLASS_AUDIO_CLOSED                 0
#define CUTLASS_AUDIO_HALFOPEN               1
#define CUTLASS_AUDIO_OPEN                   2

typedef struct cutlass_audio_req {
  uint8_t type;
  uint8_t flags;
  uint8_t encoders; 
} cutlass_audio_req_t;

#define CUTLASS_AUDIO_REQ_LEN      3

/* cutlass audio INFO packet */
/* #define CUTLASS_AUDIO_INFO_STAT             1 */
/* //// The length of a audio_info message (packet) */
/* #define CUTLASS_AUDIO_INFO_PSIZE(a)  (sizeof(uint8_t) * (3 + a)) */

/* typedef struct cutlass_audio_info { */
/*   uint8_t   type; */
/*   uint8_t   flags; */
/*   uint8_t data_len; */
/*   uint8_t *data; */
/*   struct cutlass_packet_hdr *cutlass_hdr;  */
/*   uint16_t  payload_len; */
/*   uint8_t   *payload; */
/* } cutlass_audio_info_t; */


/* /\* cutlass audio DATA packet *\/ */
/* #define CUTLASS_AUDIO_DATA_NORMAL           1 */
/* //// The length of a audio_data message (packet) */
/* #define CUTLASS_AUDIO_DATA_PSIZE(a) (sizeof(uint8_t) * (3 + a)) */


/* typedef struct cutlass_audio_data { */
/*   uint8_t   type; */
/*   uint16_t  seq; */
/*   uint16_t  data_len; */
/*   uint8_t   *data; */
/*   struct cutlass_packet_hdr *cutlass_hdr; */
/*   uint16_t  payload_len; // after packing the whole packet is in *payload */
/*   uint8_t   *payload; */
/* } cutlass_audio_data_t; */

#endif /* CUTLASS_AUDIO_TYPES_H */
