/*
  Crypto layer for Cutlass
  (C) 2004 Jack Lloyd <lloyd@randombit.net>
  
  Note that a couple of these functions don't exist at all in crypto.cpp yet,
  and more are no-ops, as I'm not sure exactly what we're going to need. We need
  to sit down and have a nice discussion about PMTU discovery and UDP at some
  point, as well. I think that will affect the handshaking code a lot, and we
  should be sure we know what we're doing before we do it.
  
  I am not terribly in love with what's in this header file. Honestly, this
  stuff is junk. I'm just tossing out code and waiting for comments, basically.
*/

#ifndef CUTLASS_CRYPTO_H__
#define CUTLASS_CRYPTO_H__

/* Link all functions as C functions, not C++ */
#ifdef __cplusplus
extern "C" {
#endif
  
#include <inttypes.h>
#include <stdio.h>
  
#define CIPHER_ALGO_AES128 1
#define CIPHER_ALGO_AES256 2

#define MAC_ALGO_HMACSHA1 1

#define DH_GROUP_1024 1
#define DH_GROUP_2048 2

/* Init/shutdown functions for the crypto stuff

   Might want to tie this together better; for example it is my understanding
   that under Windows you have to run some initilization code to use sockets.
   Ie, have a general cutlass_init() that does all this crap in one go, and a
   similiar shutdown. For later, once we've figured out what-all we need, I
   guess.
*/
void cutlass_crypto_init(void);
void cutlass_crypto_shutdown(void);

/******************************************************************
* Various types related to the crypto code                        *
******************************************************************/
/* This is opaque to prevent details of the crypto library from being visible
   outside the crypto module. In addition, since (currently) we're using a C++
   crypto library, doing otherwise would require that anything that includes
   crypto.h be compiled as C++. A certain C fan wouldn't like that. :)

   This actually does keep things cleaner in the end. I think.  Well, we can
   revisit later if it seems like it's an issue.

   These are basically opaque types. Don't be touching the bits inside of them
   outside of crypto.cpp
*/
struct cutlass_keyset_struct
   {
   /* these are set up with the keys for sending stuff */
   void* opaque_mac_send;
   void* opaque_cipher_send;
   void* opaque_cipher_filter_send;

   /* these are set up with the keys for receiving stuff */
   void* opaque_mac_recv;
   void* opaque_cipher_recv;
   void* opaque_cipher_filter_recv;

   void* send_proc;
   void* recv_proc;
   };

struct cutlass_public_key_struct
   {
   void* key;
   };

struct cutlass_private_key_struct
   {
   void* key;
   };

typedef struct cutlass_public_key_struct cutlass_public_key;
typedef struct cutlass_private_key_struct cutlass_private_key;
typedef struct cutlass_keyset_struct cutlass_keyset;

/******************************************************************
* Functions to create / save / load / destroy private keys        *
******************************************************************/

/* Load a passphrase, calling passphrase_func to get the passphrase.
   passphrase_func is not called if the key in the file is not encrypted.

   First arg of passphrase_func is something mildly descriptive (the name of
   the file, if load_private_key is being used), the second is the buffer to
   put it in, the last is the max size of the buffer. This would be much
   cleaner with std::string. :)

   passphrase_func returns 0 if OK, 1 if the operation was cancelled by the
   user.
*/
typedef int (*passphrase_func)(const char* description, char[], uint32_t);

cutlass_private_key load_private_key(const char* a_file, passphrase_func);
cutlass_public_key load_public_key(const uint8_t* raw_key);

/* If passphrase is NULL, don't encrypt. Otherwise, encrypt the key with
   the passphrase, and store the (PEM-encoded) output into out) */
void save_private_key(cutlass_private_key, const char* passphrase, FILE* out);
void save_public_key(cutlass_public_key, FILE* out);

cutlass_public_key copy_public_key(cutlass_public_key);

void free_public_key(cutlass_public_key);
void free_private_key(cutlass_private_key);

cutlass_private_key generate_dh_key(uint8_t group); /* DH_GROUP_* */

typedef void (*callback_fn)(void*);

cutlass_private_key generate_rsa_key(uint32_t size_in_bits,
                                     callback_fn callback, void* callback_arg);

cutlass_public_key extract_public_key(cutlass_private_key);

/* need functions for saving/loading/exporting public keys */

/******************************************************************
* Actual cryptographic operations                                 *
******************************************************************/
void file_hash(int fd, uint8_t hash_result[32]);

/* Return value is malloc'ed */
char* fingerprint_key(cutlass_public_key);

void fingerprint_key_bin(cutlass_public_key, uint8_t hash[32]);

cutlass_keyset empty_keyset();
void free_keyset(cutlass_keyset);

#ifdef __cplusplus
}
#endif

#endif
