/*
 * kex.h
 *
 * The key exchange stuff, lives in kex.cpp
 *
 * Copyright (c) 2004 Jack Lloyd <lloyd@randombit.net>
 *
 */

/*
   WARNING: This file can only be included from C++ code

*/

#ifndef CUTLASS_KEX_H__
#define CUTLASS_KEX_H__

#ifndef __cplusplus
  #error cutlass/kex.h can only be included from C++ code
#endif

#include <botan/rsa.h>
#include <botan/dh.h>
using namespace Botan; /* lazy */

#include <map>

#include <cutlass.h>

/*
   Magical DH groups
*/
enum DH_GROUP { DH_1024 = 0, DH_1536 = 1 };

enum PACKET_TYPE {
   CLIENT_HELLO = CUT_KEX_CHELLO,
   SERVER_HELLO = CUT_KEX_SHELLO,
   CLIENT_KEX   = CUT_KEX_CDHG,
   SERVER_KEX   = CUT_KEX_SDHG
};

enum CIPHER_TYPE {
   CIPHER_AES_128       = 1,
   CIPHER_AES_256       = 2
};

/* This is shared between HMAC + signature hashes for simplicity,
   but we always reject SHA-1 for signatures.
*/
enum HASH_TYPE {
   HASH_SHA160           = 1,
   HASH_SHA256           = 2,
   HASH_SHA512           = 3,
};

/* These are named similiar to TLS handshake messages, mostly because I've 
   been working on a TLS implementation. :) They are slightly related to the
   TLS handshake messages in purpose, but only slightly. -JL
*/

/*************************************************
* Generic Handshake message                      *
*************************************************/
class Handshake_Message
   {
   public:
      virtual SecureVector<byte> serialize() const = 0;
      virtual PACKET_TYPE type() const = 0;
      virtual ~Handshake_Message() {}
   };

/*************************************************
* ClientHello                                    *
*************************************************/
class ClientHello : public Handshake_Message
   {
   public:
      SecureVector<byte> serialize() const;

      PACKET_TYPE type() const { return CLIENT_HELLO; }

      SecureVector<byte> random() const { return c_random; }
      SecureVector<byte> key_hash() const { return hash; }
      RSA_PublicKey client_key() const;

      bool hash_matches(const RSA_PublicKey&) const;

      ClientHello(const RSA_PrivateKey&);
      ClientHello(const RSA_PrivateKey&, const RSA_PublicKey&);
      ClientHello(const MemoryRegion<byte>&);
   private:
      SecureVector<byte> c_random, key, hash;
      byte version;
   };

/*************************************************
* ServerHello                                    *
*************************************************/
class ServerHello : public Handshake_Message
   {
   public:
      SecureVector<byte> serialize() const;

      PACKET_TYPE type() const { return SERVER_HELLO; }

      SecureVector<byte> random() const { return s_random; }
      SecureVector<byte> client_random() const { return c_random; }
      RSA_PublicKey server_key() const;

      CIPHER_TYPE cipher() { return cipher_algo; }
      HASH_TYPE hmac_hash() { return hmac_hash_algo; }
      HASH_TYPE sig_hash() { return sig_hash_algo; }
      
      ServerHello(const RSA_PublicKey&, const MemoryRegion<byte>&);
      ServerHello(const MemoryRegion<byte>&);
   private:
      CIPHER_TYPE cipher_algo;
      HASH_TYPE hmac_hash_algo, sig_hash_algo;
      SecureVector<byte> s_random, c_random, key;
   };

/*************************************************
* ClientKeyExchange                              *
*************************************************/
class ClientKeyExchange : public Handshake_Message
   {
   public:
      SecureVector<byte> serialize() const;

      PACKET_TYPE type() const { return CLIENT_KEX; }

      DH_PublicKey client_dh() const;
      DH_PrivateKey client_dh_priv() const;
      byte client_dh_group() const { return dh_group; }
      SecureVector<byte> client_dh_val() const { return dh_c; }

      bool check_sig(const RSA_PublicKey&, const MemoryRegion<byte>&,
                     const MemoryRegion<byte>&) const;

      ClientKeyExchange(const RSA_PrivateKey&, DH_GROUP,
                        const MemoryRegion<byte>&,
                        const MemoryRegion<byte>&);
      ClientKeyExchange(const MemoryRegion<byte>&);
   private:
      SecureVector<byte> dh_c, dh_c_priv, sig;
      byte dh_group;
   };

/*************************************************
* ServerKeyExchange                              *
*************************************************/
class ServerKeyExchange : public Handshake_Message
   {
   public:
      SecureVector<byte> serialize() const;

      PACKET_TYPE type() const { return SERVER_KEX; }

      DH_PublicKey server_dh(byte dh_group) const;
      DH_PrivateKey server_dh_priv(byte dh_group) const;
      SecureVector<byte> server_dh_val() const { return dh_s; }

      bool check_sig(const RSA_PublicKey& s_key,
                     const MemoryRegion<byte>& c_random,
                     const MemoryRegion<byte>& s_random,
                     byte group_id,
                     const MemoryRegion<byte>& dh_c) const;

      ServerKeyExchange(const RSA_PrivateKey&,
                        byte, const MemoryRegion<byte>&,
                        const MemoryRegion<byte>&,
                        const MemoryRegion<byte>&);
      ServerKeyExchange(const MemoryRegion<byte>&);
   private:
      SecureVector<byte> dh_s, dh_s_priv, sig;
   };

/*************************************************
* Network Layer                                  *
*************************************************/
class Network_Layer
   {
   public:
      void write(const Handshake_Message&, cutlass_t*, conn_t* conn);
      Handshake_Message* read(byte, const byte[], u32bit);

      void resend_last(cutlass_t*, conn_t*);

      Network_Layer() { total_frags = 0; }
   private:
      static Handshake_Message* decode_packet(byte, const byte[], u16bit);
      void write(byte, const byte[], u32bit, cutlass_t*, conn_t* conn);
      std::map<byte, SecureVector<byte> > fragments;
      SecureVector<byte> last_msg;
      byte total_frags, last_msg_type;
   };

/*************************************************
* Misc Crap Utility Functions for Kex            *
*************************************************/
DL_Group get_dh_group(byte);

#endif
